
#ifndef __BOND_GENERAL_H__
#define __BOND_GENERAL_H__

#ifdef CFG_SWITCH_LAG_INCLUDED

/* Data types definition */
typedef unsigned int        u32;
typedef signed int          s32;
typedef unsigned short      u16;
typedef unsigned short      __be16;
typedef signed short        s16;
typedef unsigned char       u8;
typedef signed char         s8;

#define bool                BOOL
#define true                TRUE
#define false               FALSE

/*
 *      IEEE 802.3 Ethernet magic constants.  The frame sizes omit the preamble
 *      and FCS/CRC (frame check sequence).
 */
#define ETH_ALEN        6               /* Octets in one ethernet addr   */
#define ETH_HLEN        14              /* Total octets in header.       */
#define ETH_ZLEN        60              /* Min. octets in frame sans FCS */
#define ETH_DATA_LEN    1500            /* Max. octets in payload        */
#define ETH_FRAME_LEN   1514            /* Max. octets in frame sans FCS */
#define ETH_FCS_LEN     4               /* Octets in the FCS             */

/* Ethernet Protocol ID */
#define ETH_P_SLOW      0x8809          /* Slow Protocol. See 802.3ad 43B */
#define ETH_TPID_TYPE   0x8100
#define IFNAMSIZ        16

/* Standard interface flags (netdevice->flags). */
#define IFF_UP          0x1             /* interface is up              */
#define IFF_BROADCAST   0x2             /* broadcast address valid      */
#define IFF_DEBUG       0x4             /* turn on debugging            */
#define IFF_LOOPBACK    0x8             /* is a loopback net            */
#define IFF_POINTOPOINT 0x10            /* interface is has p-p link    */
#define IFF_NOTRAILERS  0x20            /* avoid use of trailers        */
#define IFF_RUNNING     0x40            /* interface RFC2863 OPER_UP    */
#define IFF_NOARP       0x80            /* no ARP protocol              */
#define IFF_PROMISC     0x100           /* receive all packets          */
#define IFF_ALLMULTI    0x200           /* receive all multicast packets*/

#define IFF_MASTER      0x400           /* master of a load balancer    */
#define IFF_SLAVE       0x800           /* slave of a load balancer     */

#define IFF_MULTICAST   0x1000          /* Supports multicast           */
 
#define IFF_PORTSEL     0x2000          /* can set media type           */
#define IFF_AUTOMEDIA   0x4000          /* auto media select active     */
#define IFF_DYNAMIC     0x8000          /* dialup device with changing addresses*/

#define IFF_LOWER_UP    0x10000         /* driver signals L1 up         */
#define IFF_DORMANT     0x20000         /* driver signals dormant       */

#define IFF_ECHO        0x40000         /* echo sent packets            */

#define IFF_VOLATILE    (IFF_LOOPBACK|IFF_POINTOPOINT|IFF_BROADCAST|IFF_ECHO|\
                            IFF_MASTER|IFF_SLAVE|IFF_RUNNING|IFF_LOWER_UP|IFF_DORMANT)

/* each slave's link has 4 states */
#define BOND_LINK_UP    0               /* link is up and running */
#define BOND_LINK_FAIL  1               /* link has just gone down */
#define BOND_LINK_DOWN  2               /* link has been down for too long time */
#define BOND_LINK_BACK  3               /* link is going back */

/* The forced speed, 10Mb, 100Mb, gigabit, 2.5Gb, 10GbE. */
#define SPEED_10                10
#define SPEED_100               100
#define SPEED_1000              1000
#define SPEED_2500              2500
#define SPEED_10000             10000
#define SPEED_UNKNOWN           -1
 
/* Duplex, half or full. */
#define DUPLEX_HALF             0x00
#define DUPLEX_FULL             0x01
#define DUPLEX_UNKNOWN          0xff

#define NETDEV_TX_OK            0
#define VLAN_TAG_LEN            4

/* Byte ordering */
#define cpu_to_be16(x)          UIP_HTONS(x)

/* LACP relative definitions */
#define LACP_PRIO               5           /* priority for LACP */
#define LACP_POLLING_INTERVAL   (1000000UL) /* 1 second in us */
#define LACP_PORT_PRIORITY      0x8000
#define LACP_PORT_TIMEOUT       FALSE       /* TRUE means short LACP timeout, FALSE means long LACP timeout */

#define LACP_ACTIVE_ACTIVITY    TRUE        /* Active LACP mode */
#define LACP_PASSIVE_ACTIVITY   FALSE       /* Passive LACP mode */

#define BOND_MODE_STATIC        1
#define BOND_MODE_LACP          2

/* for web/snmp using */
#define BOND_MODE_ON            1           /* static bonding */
#define BOND_MODE_LACP_ACTIVE   2           /* dynamic bonding by active lacp */
#define BOND_MODE_LACP_PASSIVE  3           /* dynamic bonding by passive lacp */

/* Unsupported Definitions */
#define __packed
#define read_lock(x)
#define read_unlock(x)
#define queue_delayed_work
#define spinlock_t          int
#define spin_lock_init(x)      
#define spin_lock_bh(x)
#define spin_unlock_bh(x)
#define pskb_may_pull
#define rwlock_t            int
#define pr_info             sal_printf
#define pr_warning          sal_printf 
#define pr_err              sal_printf 

#ifndef htons
#define htons               uip_htons
#endif

#ifndef ntohs
#define ntohs               uip_ntohs
#endif

#if FOXCONN_MALLOC
#define malloc(x)           fox_malloc(x, memnode_lacp)
#define free                fox_free
#else
#define malloc(x)           sal_malloc(x,"bond_malloc")
#define free                sal_free
#endif

#ifndef memset
#define memset(x,y,z)  sal_memset(x,y,z)
#endif

#ifndef memcpy
#define memcpy(x,y,z)  sal_memcpy(x,y,z)
#endif

#ifndef memcmp
#define memcmp(x,y,z)  sal_memcmp(x,y,z)
#endif

#ifndef strcmp
#define strcmp lib_strcmp
#endif

#ifndef strlen
#define strlen lib_strlen
#endif

#ifndef strcpy
#define strcpy lib_strcpy
#endif

#ifndef strncpy
#define strncpy lib_strncpy
#endif

#ifndef strstr
#define strstr sal_strstr
#endif

#ifndef atoi
#define atoi lib_atoi
#endif

enum netdev_state_t
{
    __LINK_STATE_XOFF=0,
    __LINK_STATE_START,
    __LINK_STATE_PRESENT,
    __LINK_STATE_SCHED,
    __LINK_STATE_NOCARRIER,
    __LINK_STATE_RX_SCHED,
    __LINK_STATE_LINKWATCH_PENDING
};

#define ETH_NAME    "eth"

#pragma pack(1)

struct mac_header {
    unsigned char   dst_mac[ETH_ALEN];
    unsigned char   src_mac[ETH_ALEN];
};

struct net_device {

    u16                 uport;              /* logical port (0 based) */
    char                name[IFNAMSIZ];     /* interface name */

    unsigned short      flags;              /* interface flags, ex: IFF_UP */
    //unsigned short      state;              /* netdev_state_t */
    unsigned char       dev_addr[ETH_ALEN]; /* hw address, (before bcast because most packets are unicast) */
    struct bonding      *bond;              /* private data for bonding */
    
    struct net_device   *master;            /* Pointer to master device of a group, which this device is member of. */
};

struct sk_buff {

    __be16              protocol;
    unsigned int        len;

    uint8               pdu_type;   /* 1: lacpdu, 2: marker pdu */
    void                *data;
};

struct lacp_enslave_cfg {
    BOOL    cfgInitial;     /* TRUE for cfg load to enslave all member ports */
    uint8   u8LagID;        /* LAG ID */
    uint16  uport_from;     /* Start logical port */
    uint16  uport_to;       /* End logical port (should bigger than lport_from) */
    BOOL    lacp_fast;      /* TRUE means short LACP timeout, FALSE means long LACP timeout */
    BOOL    lacp_activity;  /* TRUE means Active LACP, FALSE means Passive LACP */
    uint16  port_priority;  /* Port priority */
};

#pragma pack()

#endif /* CFG_SWITCH_LAG_INCLUDED */
#endif //__BOND_GENERAL_H__

