/*
 * Bond several ethernet interfaces into a Cisco, running 'Etherchannel'.
 *
 * Portions are (c) Copyright 1995 Simon "Guru Aleph-Null" Janes
 * NCM: Network and Communications Management, Inc.
 *
 * BUT, I'm the one who modified it for ethernet, so:
 * (c) Copyright 1999, Thomas Davis, tadavis@lbl.gov
 *
 *	This software may be used and distributed according to the terms
 *	of the GNU Public License, incorporated herein by reference.
 *
 */

#ifndef _LINUX_BONDING_H
#define _LINUX_BONDING_H

#if 0
#include <linux/timer.h>
#include <linux/proc_fs.h>
#include <linux/if_bonding.h>
#include <linux/cpumask.h>
#include <linux/in6.h>
#include <linux/netpoll.h>
#include "bond_3ad.h"
#include "bond_alb.h"
#else
#include "appl/lacp/bond_general.h"
#include "appl/lacp/bond_3ad.h"
#endif

#ifdef CFG_SWITCH_LAG_INCLUDED

#define LACP_STATE_INDEP    "indep"
#define LACP_STATE_DOWN     "down"
#define LACP_STATE_BNDL     "bndl"
#define LACP_STATE_HOTSBY   "hot-sby"

#define DEFAULT_LACP_SYS_PRIORITY   0x8000
#define DEFAULT_LACP_PORT_PRIORITY  0x8000

#define LACP_ERR(x) do { sal_printf("LACP: "); sal_printf x; } while(0);

/* for debug trace messages */
#define pr_debug(fmt, ...)                          \
{                                                   \
        if(lacp_dbg_print_get())                     \
          sal_printf("[LACP] %04d %s: " fmt,    \
          __LINE__,__FUNCTION__, ## __VA_ARGS__);   \
}

#if 1
/* FIXME ! */
#define IS_UP(dev) (((dev)->flags & IFF_UP) == IFF_UP)		   

/*
 * Checks whether slave is ready for transmit.
 */
#define SLAVE_IS_OK(slave)			        \
		    (((slave)->dev->flags & IFF_UP)  && \
		     ((slave)->link == BOND_LINK_UP) && \
		     bond_is_active_slave(slave))
#else

/**
 *      netif_running - test if up
 *      @dev: network device
 *
 *      Test if the device has been brought up.
 */
static inline bool netif_running(const struct net_device *dev)
{
    return test_bit(__LINK_STATE_START, &dev->state);
}


#define IS_UP(dev)					   \
	      ((((dev)->flags & IFF_UP) == IFF_UP)	&& \
	       netif_running(dev)			&& \
	       netif_carrier_ok(dev))

/*
 * Checks whether slave is ready for transmit.
 */
#define SLAVE_IS_OK(slave)			        \
		    (((slave)->dev->flags & IFF_UP)  && \
		     netif_running((slave)->dev)     && \
		     ((slave)->link == BOND_LINK_UP) && \
		     bond_is_active_slave(slave))
#endif


/**
 * bond_for_each_slave_from - iterate the slaves list from a starting point
 * @bond:	the bond holding this list.
 * @pos:	current slave.
 * @cnt:	counter for max number of moves
 * @start:	starting point.
 *
 * Caller must hold bond->lock
 */
#define bond_for_each_slave_from(bond, pos, cnt, start)	\
	for (cnt = 0, pos = start;				\
	     cnt < (bond)->slave_cnt;				\
             cnt++, pos = (pos)->next)

/**
 * bond_for_each_slave_from_to - iterate the slaves list from start point to stop point
 * @bond:	the bond holding this list.
 * @pos:	current slave.
 * @cnt:	counter for number max of moves
 * @start:	start point.
 * @stop:	stop point.
 *
 * Caller must hold bond->lock
 */
#define bond_for_each_slave_from_to(bond, pos, cnt, start, stop)	\
	for (cnt = 0, pos = start;					\
	     ((cnt < (bond)->slave_cnt) && (pos != (stop)->next));	\
             cnt++, pos = (pos)->next)

/**
 * bond_for_each_slave - iterate the slaves list from head
 * @bond:	the bond holding this list.
 * @pos:	current slave.
 * @cnt:	counter for max number of moves
 *
 * Caller must hold bond->lock
 */
#define bond_for_each_slave(bond, pos, cnt)	\
		bond_for_each_slave_from(bond, pos, cnt, (bond)->first_slave)

/**
 * is_zero_ether_addr - Determine if give Ethernet address is all zeros.
 * @addr: Pointer to a six-byte array containing the Ethernet address
 *
 * Return true if the address is all zeroes.
 */
static inline bool is_zero_ether_addr(const u8 *addr)
{
    return !(addr[0] | addr[1] | addr[2] | addr[3] | addr[4] | addr[5]);
}

#pragma pack(1)

struct ad_info {
    u16 aggregator_id;
    u16 ports;
    u16 actor_key;
    u16 partner_key;
    u8  partner_system[ETH_ALEN];
};

struct slave {
	struct net_device *dev; /* first - useful for panic debug */
	struct slave *next;
	struct slave *prev;
	struct bonding *bond; /* our master */
	//int    delay;
	//unsigned long   jiffies;
	//unsigned long   last_arp_rx;
	s8     link;        /* one of BOND_LINK_XXXX */
	//s8     new_link;
	
	u8     backup:1,    /* indicates backup slave. Value corresponds with BOND_STATE_ACTIVE and BOND_STATE_BACKUP */
	       inactive:1;  /* indicates inactive slave */
	u8     duplex;
	//u32    original_mtu;
	//u32    link_failure_count;
	u32    speed;
	//u16    queue_id;
	u8     perm_hwaddr[ETH_ALEN];   /* slave's original ("permanent") mac address */
	struct ad_slave_info ad_info;   /* HUGE - better to dynamically alloc */
    /*
	struct tlb_slave_info tlb_info;
#ifdef CONFIG_NET_POLL_CONTROLLER
	struct netpoll *np;
#endif
    */
};

struct bonding {
	struct      net_device *dev;     /* first - useful for panic debug */
	struct      slave *first_slave;
	//struct      slave *curr_active_slave;
	//struct      slave *current_arp_slave;
	//struct      slave *primary_slave;
	//bool        force_primary;
	u8          slave_cnt;          /* never change this value outside the attach/detach wrappers */
	//void        (*recv_probe)(struct sk_buff *, struct bonding *, struct slave *);
	rwlock_t    lock;
    #if 0    
    rwlock_t    curr_slave_lock;
	s8          kill_timers;

	u8	        send_peer_notif;
	s8	        setup_by_slave;
	s8          igmp_retrans;
	
#ifdef CONFIG_PROC_FS
	struct      proc_dir_entry *proc_entry;
	char        proc_file_name[IFNAMSIZ];
#endif /* CONFIG_PROC_FS */
    #endif
    
	//struct      list_head bond_list;
	//struct      netdev_hw_addr_list mc_list;
	int         (*xmit_hash_policy)(struct sk_buff *, int);
	//__be32      master_ip;
	//u16         flags;
	//u16         rr_tx_counter;
	struct      ad_bond_info ad_info;

#if 0
	struct   alb_bond_info alb_info;
	struct   bond_params params;
	struct   list_head vlan_list;
	struct   vlan_group *vlgrp;
	struct   packet_type arp_mon_pt;
	struct   workqueue_struct *wq;
	struct   delayed_work mii_work;
	struct   delayed_work arp_work;
	struct   delayed_work alb_work;
	struct   delayed_work ad_work;
	struct   delayed_work mcast_work;
	
#ifdef CONFIG_DEBUG_FS
	/* debugging suport via debugfs */
	struct	 dentry *debug_dir;
#endif /* CONFIG_DEBUG_FS */
#endif
};

/* bonding node for lacp */
struct bond_node {
    uint8   u8enable;                   /* TRUE: LAG enable, FALSE: LAG disable */
    BOOL    blActivity;                 /* TRUE: active mode, FALSE: passive mode */
    uint8   uplist[sizeof(pbmp_t)];     /* port bitmap */
    struct  bonding *pBond;             /* bonding pointer */
};

struct lacp_bond {
    struct ad_system lacp_sys_info;
    struct bond_node lacp_channel[BOARD_MAX_NUM_OF_LAG];
};

struct lacp_port_info {
    uint16 uport;//lport;
    struct port_params actor_oper;
    struct port_params partner_oper;
    char szLacpActorState[8];
};

struct port_channel_info {
    uint8 u8Ports;
    struct lacp_port_info *pLacp_port_info;    
};

#pragma pack()

static inline struct bonding *netdev_priv(struct net_device *dev)
{
    return((struct bonding *)dev->bond);
}

static inline void bond_set_active_slave(struct slave *slave)
{
	slave->backup = 0;
}

static inline void bond_set_backup_slave(struct slave *slave)
{
	slave->backup = 1;
}

static inline int bond_slave_state(struct slave *slave)
{
	return slave->backup;
}

static inline bool bond_is_active_slave(struct slave *slave)
{
	return !bond_slave_state(slave);
}

static inline void bond_set_slave_inactive_flags(struct slave *slave)
{
#if 0
    struct bonding *bond = netdev_priv(slave->dev->master);
    
	if (!bond_is_lb(bond))
		bond_set_backup_slave(slave);

	if (!bond->params.all_slaves_active)
		slave->inactive = 1;
#else
    bond_set_backup_slave(slave);
    slave->inactive = 1;
#endif
}

static inline void bond_set_slave_active_flags(struct slave *slave)
{
	bond_set_active_slave(slave);
	slave->inactive = 0;
}

static inline bool bond_is_slave_inactive(struct slave *slave)
{
	return slave->inactive;
}

static inline struct bonding *bond_get_bond_by_slave(struct slave *slave)
{
	if (!slave || !slave->dev->master) {
		return NULL;
	}

	return netdev_priv(slave->dev->master);
}

int bond_xmit_hash_policy_l2(struct sk_buff *skb, int count);
int bond_dev_queue_xmit(struct bonding *bond, struct sk_buff *skb, struct net_device *slave_dev);
void lacp_linkchange(uint16 uport, BOOL link, void *arg);
void lacp_dbg_print_set(BOOL blflag);
BOOL lacp_dbg_print_get(void);
void lacp_init_cfg(struct lacp_enslave_cfg *placp_cfg);
int lacp_is_enabled(uint8 u8LagID, uint8 *pMemberPorts);
BOOL lacp_is_aggregator_active(uint8 u8LagID);
int lacp_is_LAG_available(uint8 u8BondMode, struct lacp_enslave_cfg *pLacp_cfg);
uint8 lacp_check_lag_type(uint16 uport);
uint8 lacp_get_active_lag(unsigned int nIdx, uint8 *pBondMode);
uint32 lacp_get_member_ports(uint8 u8LagID, uint8 lag_lplist[sizeof(pbmp_t)]);
BOOL lacp_get_port_mapping(uint8 u8BondMode, uint8 u8LagID, uint16 lport_from, uint16 lport_to, uint8 *pCur_lplist, uint8 *pDel_lplist);
BOOL lacp_get_bond_slave_by_lport(uint16 lport, uint8 *pGetLagID, struct bonding **pBond, struct slave **pSlave);
BOOL lacp_bond_get_channel_info(uint8 u8LagID, struct port_channel_info *pChannelInfo);
int lacp_bond_enslave(struct lacp_enslave_cfg *pLacp_cfg);
int lacp_bond_release(BOOL blDefaulted, uint8 u8LagID, uint16 lport_from, uint16 lport_to);
int lacp_bond_release_all(BOOL blDefaulted, uint8 u8LagID);
void lacp_set_sys_priority(uint16 sys_priority);
int lacp_set_port_priority(uint16 uport, uint16 port_priority);
void lacp_get_bond_info(struct lacp_bond *pLacpBondInfo);
BOOL lacp_enable_set(uint8 enable);
void lacp_init(void);

#endif /* CFG_SWITCH_LAG_INCLUDED */
#endif /* _LINUX_BONDING_H */
