/*
 * $Id: l2_addr.h 1.4 Broadcom SDK $
 *
 * $Copyright: Copyright 2012 Broadcom Corporation.
 * This program is the proprietary software of Broadcom Corporation
 * and/or its licensors, and may only be used, duplicated, modified
 * or distributed pursuant to the terms and conditions of a separate,
 * written license agreement executed between you and Broadcom
 * (an "Authorized License").  Except as set forth in an Authorized
 * License, Broadcom grants no license (express or implied), right
 * to use, or waiver of any kind with respect to the Software, and
 * Broadcom expressly reserves all rights in and to the Software
 * and all intellectual property rights therein.  IF YOU HAVE
 * NO AUTHORIZED LICENSE, THEN YOU HAVE NO RIGHT TO USE THIS SOFTWARE
 * IN ANY WAY, AND SHOULD IMMEDIATELY NOTIFY BROADCOM AND DISCONTINUE
 * ALL USE OF THE SOFTWARE.  
 *  
 * Except as expressly set forth in the Authorized License,
 *  
 * 1.     This program, including its structure, sequence and organization,
 * constitutes the valuable trade secrets of Broadcom, and you shall use
 * all reasonable efforts to protect the confidentiality thereof,
 * and to use this information only in connection with your use of
 * Broadcom integrated circuit products.
 *  
 * 2.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, THE SOFTWARE IS
 * PROVIDED "AS IS" AND WITH ALL FAULTS AND BROADCOM MAKES NO PROMISES,
 * REPRESENTATIONS OR WARRANTIES, EITHER EXPRESS, IMPLIED, STATUTORY,
 * OR OTHERWISE, WITH RESPECT TO THE SOFTWARE.  BROADCOM SPECIFICALLY
 * DISCLAIMS ANY AND ALL IMPLIED WARRANTIES OF TITLE, MERCHANTABILITY,
 * NONINFRINGEMENT, FITNESS FOR A PARTICULAR PURPOSE, LACK OF VIRUSES,
 * ACCURACY OR COMPLETENESS, QUIET ENJOYMENT, QUIET POSSESSION OR
 * CORRESPONDENCE TO DESCRIPTION. YOU ASSUME THE ENTIRE RISK ARISING
 * OUT OF USE OR PERFORMANCE OF THE SOFTWARE.
 * 
 * 3.     TO THE MAXIMUM EXTENT PERMITTED BY LAW, IN NO EVENT SHALL
 * BROADCOM OR ITS LICENSORS BE LIABLE FOR (i) CONSEQUENTIAL,
 * INCIDENTAL, SPECIAL, INDIRECT, OR EXEMPLARY DAMAGES WHATSOEVER
 * ARISING OUT OF OR IN ANY WAY RELATING TO YOUR USE OF OR INABILITY
 * TO USE THE SOFTWARE EVEN IF BROADCOM HAS BEEN ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGES; OR (ii) ANY AMOUNT IN EXCESS OF
 * THE AMOUNT ACTUALLY PAID FOR THE SOFTWARE ITSELF OR USD 1.00,
 * WHICHEVER IS GREATER. THESE LIMITATIONS SHALL APPLY NOTWITHSTANDING
 * ANY FAILURE OF ESSENTIAL PURPOSE OF ANY LIMITED REMEDY.$
 *
 */

#ifndef _BOARDAPI_L2_ADDR_H_
#define _BOARDAPI_L2_ADDR_H_

#ifdef CFG_SWITCH_L2_ADDR_INCLUDED

#define L2_FDB_MAX_SIZE_WOLF  16384
#define BOARD_L2_STATIC   (0x1 << 0)

typedef enum l2_cmd_s {
    L2_CMD_ADD = 0,
    L2_CMD_DELETE,
    L2_CMD_DELETE_BY_PORT,
    L2_CMD_DELETE_BY_VLAN,
    L2_CMD_DELETE_BY_TRUNK,
    L2_CMD_DELETE_ALL,
    L2_CMD_LOOKUP,
    L2_CMD_GET_FIRST,
    L2_CMD_GET_NEXT,
    L2_CMD_GET_LAST,
    L2_CMD_GET_ALL,
    L2_CMD_COUNT
} l2_cmd_t;

typedef struct board_l2_addr_s {
  uint16 flags;   /* BOARD_L2_xxx flags. */
  uint8  mac[6];  /* 802.3 MAC address. */
  uint16 vid;     /* VLAN identifier. */
  uint16 uport;   /* user port number. */
  uint16 mcidx;   /* L2MC pointer. */
  uint8  tgid;    /* Trunk ID value. */
  uint8  is_trunk;  /* Trunk bit. */
  uint8  is_mc;   /* L2MC */
} board_l2_addr_t;


extern sys_error_t board_l2_addr_add(board_l2_addr_t *l2addr);

extern sys_error_t board_l2_addr_delete(board_l2_addr_t *l2addr);

extern sys_error_t board_l2_addr_delete_by_port(uint16 uport);

extern sys_error_t board_l2_addr_delete_by_vlan(uint16 vid);

extern sys_error_t board_l2_addr_delete_by_trunk(uint8 lagid);

extern sys_error_t board_l2_addr_delete_all(void);

extern sys_error_t board_l2_addr_lookup(board_l2_addr_t *l2addr, 
                                                         uint16 *index);

extern sys_error_t board_l2_addr_get_first(board_l2_addr_t *l2addr, 
                                                            uint16 *index);

extern sys_error_t board_l2_addr_get_next(board_l2_addr_t *l2addr, 
                                                            uint16 *index);

extern sys_error_t board_l2_addr_get_last(board_l2_addr_t *l2addr, 
                                                            uint16 *index);

extern sys_error_t board_l2_addr_get(uint8 *mac_addr, uint16 *vlan_id, uint16 *uport, uint8 *s_flag, uint8 *t_flag, int *index);

extern int board_l2_addr_count_get(void);

extern sys_error_t board_l2_static_uc_add(uint8 *mac_addr, uint16 vlan_id, uint16 uport);

extern sys_error_t board_l2_static_uc_remove(uint8 *mac_addr, uint16 vlan_id, uint16 uport);

extern sys_error_t board_l2_addr_flush_by_system(void);

extern sys_error_t board_l2_addr_flush_by_port(uint16 uport, uint8 s_flag);

extern sys_error_t board_l2_addr_flush_by_lagid(uint8 lagid, uint8 s_flag);

extern void board_l2_addr_linkchange(uint16 uport, BOOL link, void *arg) REENTRANT;


#endif  /* CFG_SWITCH_L2_ADDR_INCLUDED */

#endif /* _BOARDAPI_L2_ADDR_H_ */
