#include "system.h"
//#include "utils/sys.h"
#include "utils/ports.h"
#include "board.h"
#include "bsp_config.h"
#include "brdimpl.h"
#include "soc/bcm5340x.h"
#include "boardapi/port.h"
#include "boardapi/port.h"
//#include "boardapi/stp.h"
#include "boardapi/lag.h"
#include "boardapi/l2_addr.h"
//#include "appl/rstp/rstp.h"
#include "appl/lacp/bonding.h"

//extern soc_switch_t soc_switch_bcm5333x;

#ifdef CFG_SWITCH_LAG_INCLUDED
static void board_lag_fwdport_set(uint8 tgid, pbmp_t pbmp);
static void board_lag_respport_set(uint8 tgid, pbmp_t pbmp);

/* variables for static trunk */
static uint8 lag_enable = FALSE;
static uint8 lag_mode = DEFAULT_TRUNK_RTAG;       /* default load balance mode */
static lag_group_t lag_group[BOARD_MAX_NUM_OF_LAG];


STATIC SYS_LAGCHANGE_FUNC lagchange_handlers[CFG_MAX_REGISTERED_LAGCHANGE];

void 
APIFUNC(lagchange_init)(void) REENTRANT
{
    uint8 nIdx;

    for(nIdx=0; nIdx<CFG_MAX_REGISTERED_LAGCHANGE; nIdx++) {
        lagchange_handlers[nIdx] = NULL;
    }
}

BOOL 
APIFUNC(sys_register_lagchange)(SYS_LAGCHANGE_FUNC func) REENTRANT
{
    uint8 nIdx;
    
    SAL_ASSERT(func != NULL);
    SAL_DEBUGF(("sys_register_lagchange: %p\n", func));

    for(nIdx=0; nIdx<CFG_MAX_REGISTERED_LAGCHANGE; nIdx++) {
        if (lagchange_handlers[nIdx] == NULL) {
            lagchange_handlers[nIdx] = func;
            return TRUE;
        }
    }
    
    return FALSE;
}

void 
APIFUNC(sys_unregister_lagchange)(SYS_LAGCHANGE_FUNC func) REENTRANT
{
    uint8 nIdx;
    
    SAL_ASSERT(func != NULL);
    SAL_DEBUGF(("sys_unregister_lagchange: %p\n", func));

    for(nIdx=0; nIdx<CFG_MAX_REGISTERED_LAGCHANGE; nIdx++) {
        if (lagchange_handlers[nIdx] == func) {
            lagchange_handlers[nIdx] = NULL;
            return;
        }
    }
}

static sys_error_t board_lag_update_hw_pbmp(pbmp_t *hw_lpbmp)
{
    BOOL link;
    uint8 lport, unit, lagid, u8Idx;
    uint16 uport;
    uint16  nMaxPorts = board_uport_count();
    struct port_channel_info channelInfo;
    uint8 lag_uplist[MAX_UPLIST_WIDTH];

    if (*hw_lpbmp==0){
        return SYS_OK;
    }

    /* remove link down port from hw_pbmp */
    for (uport=1; uport<=nMaxPorts; uport++)
    {
        if (SYS_OK==board_port_link_status_get(uport, &link)) {
            if (!link) {
                if (SYS_OK==board_uport_to_lport(uport, &unit, &lport)) {
                    *hw_lpbmp &= ~(0x1 << lport);
                    continue;
                }
            }
        }
    }

    board_lpbmp_to_uplist(0, *hw_lpbmp, lag_uplist);
    
    /* remove the LACP port which is not in the bndl state from hw_pbmp */
    for (lagid = 1 ; lagid <= BOARD_MAX_NUM_OF_LAG ; lagid++) {
        if (TRUE==lacp_bond_get_channel_info(lagid, &channelInfo)) {
            for(u8Idx=0; u8Idx<channelInfo.u8Ports; u8Idx++) {                
                uport= channelInfo.pLacp_port_info[u8Idx].uport;
                if (SYS_OK != uplist_port_matched(lag_uplist, channelInfo.pLacp_port_info[u8Idx].uport))
                    break;
                if (memcmp(channelInfo.pLacp_port_info[u8Idx].szLacpActorState, LACP_STATE_BNDL, strlen(LACP_STATE_BNDL))){
                    if (SYS_OK==board_uport_to_lport(uport, &unit, &lport)) {
                        *hw_lpbmp &= ~(0x1 << lport);
                    }
                    
                }
            }
            free(channelInfo.pLacp_port_info);
        }
    }

    return SYS_OK;
}


int
board_lag_serializer_size_get(void)
{
    int nStaticBondSize, nDynamicBondSize,portPrioritySize;

    nStaticBondSize = sizeof(lag_enable) + sizeof(lag_mode) + ((sizeof(uint8) + sizeof(pbmp_t)) * BOARD_MAX_NUM_OF_LAG);
    nDynamicBondSize = sizeof(uint16) + ((sizeof(uint8)*2 + sizeof(pbmp_t)) * BOARD_MAX_NUM_OF_LAG);
    portPrioritySize = sizeof(uint16) * board_uport_count();
    return (nStaticBondSize + nDynamicBondSize+portPrioritySize);
}

/* Setup the lag to enable/disable. To unify the LAG settings by calling the board_lag_group_set() */
sys_error_t
board_lag_set(uint8 enable)
{
    if (lag_enable != enable) {
        
        lag_enable = enable;        
    }
    return SYS_OK;
}

void
board_lag_get(uint8 *enable)
{
    *enable = lag_enable;
}

sys_error_t
board_lag_mode_get(uint8 *mode)
{
    *mode = lag_mode;
    return SYS_OK;
}

extern void pbmp_to_uport_print(pbmp_t pbmp); 

/* Modified by peter for link aggregation mode (LAG)                                                                                        */
/* Note:                                                                                                                                    */
/*       1. The caller should use lacp_is_LAG_available() function to check the member port is not using by another LAG before to add LAG.  */
/*       2. It's reasonable to add/delete one member port in the existing trunk.                                                            */
/*       3. Do not check the member port, since the port speed/duplex can be changed to different value later.                              */
/*       4. Only support LAG with default vlan in current version.                                                                          */
sys_error_t 
board_lag_group_set(uint8 lagid, uint8 enable, uint8 *uplist, uint8 rtag)
{
    pbmp_t lpbmp;
    pbmp_t hw_lpbmp, org_cfg_lpbmp;
    uint8  i, count = 0, menable;
    uint8  lport;
    BOOL   mmode=FALSE;
    uint16 uport, tmp_uport, mirror_uport, org_respport = 0;
    uint8  org_uplist[MAX_UPLIST_WIDTH];

    if ((lagid == 0) || (lagid > BOARD_MAX_NUM_OF_LAG)) {
        return SYS_ERR_PARAMETER;
    }
    
    if (uplist != NULL) {
        SOC_IF_ERROR_RETURN(board_uplist_to_lpbmp(uplist, 0, &lpbmp));
    } else {
        return SYS_ERR_PARAMETER;
    }
    

    board_lag_respport_get(LAGID2TGID(lagid), &org_respport);
    board_mirror_to_get(&mirror_uport);

    /*get mirror setting,if any port is mirrored, the mirror is enable*/
    SAL_UPORT_ITER(tmp_uport) {
        board_mirror_port_get((uint16)tmp_uport, &menable);
        if (TRUE == menable)
            break;
    }

    if (TRUE == menable)
        mmode = TRUE;
    
    //sal_printf("\r\n board_lag_group_set mmode %d ",mmode);


    /* Check the number of trunk member.
     * - Minimum number of ports in each trunk: 2
     * - maximum number of ports in each trunk: 8
     */
    SOC_LPORT_ITER(lport) {
    
        if(BCM_PBMP_MEMBER(lpbmp, lport)) {
        
            /* check the port is not using by port mirroring */
            if (board_lport_to_uport(0, lport, &uport)==SYS_OK) {
                if ((FALSE != mmode) && (uport==mirror_uport))
                    return SYS_ERR_STATE;
            }

            
            count ++;
            if (count > BOARD_MAX_PORT_PER_LAG) {
                return SYS_ERR_PARAMETER;
            }
        }
    }

#if 0 /* Ccc: */
    if (count == 0x1) {
        sal_printf("\r\n board_lag_group_set count %d ",count);    
        return SYS_ERR_PARAMETER;
    }
#endif
    if ((enable == TRUE) && (lag_enable == TRUE)) {
        BCM_PBMP_ASSIGN(hw_lpbmp, lpbmp);        
        board_lag_update_hw_pbmp(&hw_lpbmp);
    } else {
        BCM_PBMP_CLEAR(hw_lpbmp);    
        BCM_PBMP_CLEAR(lpbmp);    
	}

	/* HW setting */
	SOC_IF_ERROR_RETURN(bcm5340x_lag_group_set(0, (LAGID2TGID(lagid)), hw_lpbmp, rtag));

    if (enable==FALSE)
        /* Flush L2 address learned on that trunk group */
        board_l2_addr_flush_by_lagid(lagid, 0);
    
    /* lag SW database setting */
    if ((rtag >= BCM_TRUNK_RTAG_SRCMAC) && (rtag <= BCM_TRUNK_RTAG_SRCDSTIP))
        lag_mode = rtag;
    else
        lag_mode = DEFAULT_TRUNK_RTAG;    /* default value */

    org_cfg_lpbmp = lag_group[LAGID2TGID(lagid)].cfg_lpbmp;
    lag_group[LAGID2TGID(lagid)].enable = enable;
    BCM_PBMP_ASSIGN(lag_group[LAGID2TGID(lagid)].cfg_lpbmp, lpbmp);    
    BCM_PBMP_ASSIGN(lag_group[LAGID2TGID(lagid)].hw_lpbmp, hw_lpbmp);
    board_lag_fwdport_set(LAGID2TGID(lagid), hw_lpbmp);
    board_lag_respport_set(LAGID2TGID(lagid), lpbmp);


#ifdef CFG_SWITCH_LAG_INCLUDED        
    /* notify handlers if lag cfg pbmp is changed */
    if( org_cfg_lpbmp != lpbmp ) {

        /* convert to original lplist */
        memset(org_uplist, (char)0, sizeof(org_uplist));          
        board_lpbmp_to_uplist(0, org_cfg_lpbmp, org_uplist);
        
        /* try to get the new responsive port if the org_respport is 0 (new group) */
        if ((uint16)0==org_respport)
            board_lag_respport_get(LAGID2TGID(lagid), &org_respport);
        
        for(i=0; i<CFG_MAX_REGISTERED_LAGCHANGE; i++) {
            SYS_LAGCHANGE_FUNC func = lagchange_handlers[i];
            if (func != NULL) {
                (*func)(lagid, enable, org_respport, org_uplist, uplist);
            }
        }
    }
#endif  /* CFG_SWITCH_LAG_INCLUDED */

    return SYS_OK;
}

sys_error_t
board_lag_group_get(uint8 lagid, uint8 *enable, uint8 *uplist)
{
    if ((lagid == 0) || (lagid > BOARD_MAX_NUM_OF_LAG)) {
        return SYS_ERR_PARAMETER;
    }

    *enable = lag_group[LAGID2TGID(lagid)].enable;
    SOC_IF_ERROR_RETURN(board_lpbmp_to_uplist(0, lag_group[LAGID2TGID(lagid)].cfg_lpbmp, uplist));

    return SYS_OK;
}

void
board_lag_group_max_num(uint8 *num)
{
    *num = BOARD_MAX_NUM_OF_LAG;
}

/* update pbmp by enable/disable lacp member port */
void
board_lag_update_lacp_pbmp(uint16 uport, BOOL enable)
{
    uint8 i, unit, lport;
#ifdef CFG_SWITCH_STP_INCLUDED    
    uint16 resp_port;
#endif    
    pbmp_t hw_lpbmp = 0, org_hw_lpbmp = 0;
    sys_error_t rv = SYS_OK;

    /* only update the pbmp by LACP port */
    if ((TRUE==lag_enable) && (BOND_MODE_LACP==lacp_check_lag_type(uport))) {
        rv = board_uport_to_lport(uport, &unit, &lport);
        if (rv != SYS_OK) {
            return;
        }

        for (i = 1 ; i <= BOARD_MAX_NUM_OF_LAG ; i++) {
            if ((lag_group[LAGID2TGID(i)].enable == TRUE) && (BCM_PBMP_MEMBER(lag_group[LAGID2TGID(i)].cfg_lpbmp, lport))) {
                /* add lag setting to HW */
                bcm5340x_lag_group_get(0, LAGID2TGID(i), &org_hw_lpbmp);

                hw_lpbmp = org_hw_lpbmp;

                if (enable) {
                    hw_lpbmp |= (0x1 << lport);    
                } else {
                    hw_lpbmp &= ~(0x1 << lport);
                }
                bcm5340x_lag_group_set(0, LAGID2TGID(i), hw_lpbmp, lag_mode);

                lag_group[LAGID2TGID(i)].hw_lpbmp= hw_lpbmp;
                board_lag_fwdport_set(LAGID2TGID(i), hw_lpbmp);
                board_lag_respport_set(LAGID2TGID(i), lag_group[LAGID2TGID(i)].cfg_lpbmp);

                board_l2_addr_flush_by_lagid(i, 0);

#ifdef CFG_SWITCH_STP_INCLUDED
                /* for commpute lag/lacp path cost*/
                board_lag_respport_get(LAGID2TGID(i),&resp_port);
                rstp_port_cfg_init(resp_port);
#endif /* CFG_SWITCH_STP_INCLUDED */

                break;
            }
        }
    }

}

void
board_lag_linkchange(uint16 uport, BOOL link, void *arg)
{
    uint8 unit, lport;
#ifdef CFG_SWITCH_STP_INCLUDED
    uint16 resp_port;
#endif
    pbmp_t hw_lpbmp;
    uint8 i;
    sys_error_t rv = SYS_OK;

    /* only update the pbmp by static trunk port */
    if ((TRUE==lag_enable) && (BOND_MODE_STATIC==lacp_check_lag_type(uport))) {

        
        rv = board_uport_to_lport(uport, &unit, &lport);
        if (rv != SYS_OK) {
            return;
        }

        for (i = 1 ; i <= BOARD_MAX_NUM_OF_LAG ; i++) {

            if ((lag_group[LAGID2TGID(i)].enable == TRUE) && (BCM_PBMP_MEMBER(lag_group[LAGID2TGID(i)].cfg_lpbmp, lport))) {
                /* add lag setting to HW */
                bcm5340x_lag_group_get(0, LAGID2TGID(i), &hw_lpbmp);

                if (link) {
                    uint8 lag_uplist[MAX_UPLIST_WIDTH];

                    board_lpbmp_to_uplist(0, hw_lpbmp, lag_uplist);
                    
                    rv = board_lag_check_port(uport, lag_uplist);
                    if(SYS_OK==rv)
                        hw_lpbmp |= (0x1 << lport);    
                    else {
                        if (hw_lpbmp & (0x1 << lport))
                            hw_lpbmp &= ~(0x1 << lport);
                    }
                } else {
                    hw_lpbmp &= ~(0x1 << lport);
                }

                bcm5340x_lag_group_set(0, LAGID2TGID(i), hw_lpbmp, lag_mode);

                lag_group[LAGID2TGID(i)].hw_lpbmp= hw_lpbmp;
                board_lag_fwdport_set(LAGID2TGID(i), hw_lpbmp);
                board_lag_respport_set(LAGID2TGID(i), lag_group[LAGID2TGID(i)].cfg_lpbmp );

                board_l2_addr_flush_by_lagid(i, 0);

#ifdef CFG_SWITCH_STP_INCLUDED
                /* for commpute lag/lacp path cost*/
                board_lag_respport_get(LAGID2TGID(i),&resp_port);
                rstp_port_cfg_init(resp_port);
#endif                
                break;
            }
        }
    }
}

uint8
board_lag_group_number_get(void)
{
    uint8 i, lag_count=0;
	
    for( i=0; i<BOARD_MAX_NUM_OF_LAG; i++){
        if ( lag_group[i].enable )
            lag_count++;
    }

    return lag_count;
}

/* tgid: zero-based */                   
int
board_port_to_lag_group_get(uint16 uport, uint8 *tgid)
{
    int i, rv;
    uint8 enable;
    uint8 lag_uplist[MAX_UPLIST_WIDTH];
	//pbmp_t pbmp;

    /*check if the port belongs to a port trunk*/
    for (i=1; i <= BOARD_MAX_NUM_OF_LAG ; i++) {
    	rv = board_lag_group_get(i, &enable, &lag_uplist[0]);

       // board_to_physical_pbmp(lag_uplist, 0, &pbmp);

		if (rv == SYS_OK && (enable == TRUE)) {
            if (uplist_port_matched(lag_uplist, uport) == SYS_OK) {
				*tgid =(uint8)( i - 1);
				return 0;
            } 
        }
    }

    return -1 ;
}

/* Set the first fwd port in lag*/
static void board_lag_fwdport_set(uint8 tgid, pbmp_t lpbmp)
{
    uint16 uport = 0, maxPorts;
    uint8 lag_uplist[MAX_UPLIST_WIDTH];

	lag_group[tgid].fwd_port = 0;

    maxPorts = (uint16)board_uport_count();
	 
	if ((tgid < 0) || (tgid >= BOARD_MAX_NUM_OF_LAG) || (lpbmp == 0)) {
        return;
    }
	
    board_lpbmp_to_uplist(0, lpbmp, lag_uplist);
	
    for (uport = 1 ; uport <= maxPorts ; uport++) {
		//user_port_to_board(uport, &lport);
       	if (uplist_port_matched(lag_uplist, uport) == SYS_OK) {
			lag_group[tgid].fwd_port = uport;
       	    break;
       }
    }    
}	

int
board_lag_fwdport_get(uint8 tgid, uint16* fwdport)
{
	 if ((tgid < 0) || (tgid >= BOARD_MAX_NUM_OF_LAG)) {
        return SYS_ERR_PARAMETER;
    }

	*fwdport = lag_group[tgid].fwd_port;
	if (*fwdport > 0) {
		return 0;
	}	else {
		return -1;
	}
}

/* Set the first uport of cfg bpm to resp port in lag */
static void board_lag_respport_set(uint8 tgid, pbmp_t lpbmp)
{
    uint8 lag_uplist[MAX_UPLIST_WIDTH];
	uint16 uport , maxPorts;

    lag_group[tgid].resp_port = 0;
    maxPorts = (uint16)board_uport_count();

    if ((tgid < 0) || (tgid >= BOARD_MAX_NUM_OF_LAG) || (lpbmp == 0)) {
        return;
    }

    board_lpbmp_to_uplist(0, lpbmp, lag_uplist);

	for (uport = 1; uport <= maxPorts ; uport++) {
       	if (uplist_port_matched(lag_uplist, uport) == SYS_OK) {
            lag_group[tgid].resp_port = uport;
			break;   
       } 
    }
}

int
board_lag_respport_get(uint8 tgid, uint16* respport)
{
	if ((tgid < 0) || (tgid >= BOARD_MAX_NUM_OF_LAG)) {
        return SYS_ERR_PARAMETER;
    }

	*respport = lag_group[tgid].resp_port;
	if (*respport > 0) {
		return 0;
	}	else {
		return -1;
	}
}

void
board_lag_hwpbmp_get(uint8 tgid, pbmp_t *hwlpbmp)
{
	*hwlpbmp = lag_group[tgid].hw_lpbmp;
}


uint8
board_lag_hwpbmp_cnt(uint8 tgid)
{
    uint8 cnt = 0, lag_uplist[MAX_UPLIST_WIDTH];
	uint16 uport;


    if ((tgid < 0) || (tgid >= BOARD_MAX_NUM_OF_LAG) || (lag_group[tgid].hw_lpbmp == 0)) {
        return 0;
    }

    board_lpbmp_to_uplist(0, lag_group[tgid].hw_lpbmp, lag_uplist);

	for (uport = 1; uport <= BOARD_MAX_NUM_OF_PORTS ; uport++) {
       	if (uplist_port_matched(lag_uplist, uport) == SYS_OK) {
            cnt++;
         } 
    }
    return cnt;
}

/* Use the first link up port as the master port in LAG. And validate the checkPort speed/duplex/type should be same with the master port */
sys_error_t 
board_lag_check_port(uint16 checkPort, uint8 *uplist)
{
    BOOL first_check = 1, link;
    sys_error_t rv = SYS_OK;
    uint16 uport;
    uint8 speed = 0, duplex = 0;
    uint8 all_speed = 0, all_duplex = 0;

    if ((rv = uplist_is_empty(uplist)) == SYS_OK) {

        if (board_port_speed_duplex_get(checkPort, &speed, &duplex) != SYS_OK) {
            return SYS_ERR_PARAMETER;
        }

        /* Link Aggregation is supported only on point-to-point links with MACs operating in full duplex mode. */
        if (!duplex) {
//            sal_printf("Link Aggregation is supported only on point-to-point links in full duplex mode. Wrong duplex mode on port %d\n", checkPort+1);
            return SYS_ERR_STATE;
        }
        return SYS_OK;
    }

    /* Get master port's speed/duplex/type. */
    for (uport = 1; uport <= board_uport_count() ; uport++) {

        if ((rv = uplist_port_matched(uplist, uport)) == SYS_OK)  {
            /* bypass the link down port */
            rv = board_port_link_status_get(uport, &link);
            if ((SYS_OK==rv) && (!link))
                continue;

            if (board_port_speed_duplex_get(uport, &all_speed, &all_duplex) != SYS_OK)  {
                return SYS_ERR_PARAMETER;
            }

            first_check = 0;
            break;
        }
    }

    if (!first_check) {
        
        if (board_port_speed_duplex_get(checkPort, &speed, &duplex) != SYS_OK) {
            return SYS_ERR_PARAMETER;
        }

        /* All links in a Link Aggregation Group operate at the same data rate and full duplex mode. */
        if ((speed != all_speed) || (duplex != all_duplex) || (!duplex)) 
        {
//            sal_printf("All links in a Link Aggregation Group operate at the same data rate and full duplex mode. Wrong port speed/duplex on port %d\n", checkPort+1);
            return SYS_ERR_STATE;
        }
    }

    return SYS_OK;
}

sys_error_t 
board_lag_check_vlanid(uint8 *uplist)
{
    sys_error_t rv = SYS_OK;
	int i=0, vlan_count=0; 
    uint16 vid=0;
	uint8  uplist_temp[MAX_UPLIST_WIDTH], tag_uplist_temp[MAX_UPLIST_WIDTH];
    vlan_type_t type;
    
    if (uplist == NULL)
    {
        return SYS_ERR_PARAMETER;
    }
    
    brdimpl_vlan_type_get(&type);
    vlan_count=board_vlan_count();
 
    for(i=0; i<vlan_count ; i++)   
    { 
        /* check trunk port group belong to the same member of VLANs */
        if(VT_DOT1Q==type)
        {
            rv=board_qvlan_get_by_index(i , &vid, uplist_temp, tag_uplist_temp);
        }
        else /* VT_PORT_BASED */
        {
            rv=board_pvlan_get_by_index(i , &vid, uplist_temp);
        }

        if( rv == SYS_OK)
		{
            if(uplist_manipulate(uplist_temp, uplist, UPLIST_OP_AND) != SYS_OK)
            {       
                return SYS_ERR_FALSE; 
            }

            /* 
             * 1. equal to zero, means all port group do not belong to the member of VLAN
             * 2. equal to lplist, means all port group belong to the member of VLAN
             */
            if((uplist_is_empty(uplist_temp)!=SYS_OK) && (sal_memcmp(uplist_temp, uplist, sizeof(uplist_temp))!=0))
            {
                return SYS_ERR;
            }
		}
    }
    
    return SYS_OK;    
}
#endif  /* CFG_SWITCH_LAG_INCLUDED */


